@@ -3,7 +3,7 @@ package CljPerl::Printer;
   use strict;
   use warnings;
 
-  our $VERSION = '0.04';
+  our $VERSION = '0.05';
 
   sub to_string {
     my $obj = shift;
@@ -44,6 +44,19 @@ package CljPerl::Printer;
         }
         $s .= "}";
         $s =~ s/ \}$/\}/;
+      } elsif($type eq "xml") {
+        $s = "<";
+        $s .= $obj->{name};
+        if(defined $obj->{meta}) {
+          foreach my $i (keys %{$obj->meta()}) {
+            $s .= " " . $i . "=\"" . to_string($obj->meta()->{$i}) . "\"";
+          };
+        };
+        $s .= ">";
+        foreach my $i (@{$obj->value()}) {
+          $s .= to_string($i) . " ";
+        };
+        $s .= "</" . $obj->{name} . ">";
       } elsif($type eq "function" or $type eq "macro") {
         $s = to_string($obj->value());
       } else {
